package cn.com.qmth.stmms.admin.user;

import cn.com.qmth.stmms.biz.exam.service.SubjectUserService;
import cn.com.qmth.stmms.biz.user.model.User;
import cn.com.qmth.stmms.biz.user.service.UserService;
import cn.com.qmth.stmms.biz.user.service.query.UserSearchQuery;
import cn.com.qmth.stmms.common.annotation.Logging;
import cn.com.qmth.stmms.common.annotation.RoleRequire;
import cn.com.qmth.stmms.common.controller.BaseController;
import cn.com.qmth.stmms.common.enums.LogType;
import cn.com.qmth.stmms.common.enums.Role;
import cn.com.qmth.stmms.common.enums.UserSource;
import cn.com.qmth.stmms.common.utils.EncryptUtils;
import cn.com.qmth.stmms.common.utils.RequestUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;

@Controller
@RequestMapping("/admin/user")
public class UserController extends BaseController {

    public static final String SUBJECT_CODE_SPLIT = ",";

    public static Role[] ROLE_LIST = { Role.SCANNER, Role.SUBJECT_HEADER, Role.MARKER };

    @Autowired
    private UserService userService;

    @Autowired
    private SubjectUserService subjectUserService;

    @Logging(menu = "查询用户", type = LogType.QUERY)
    @RequestMapping("/list")
    @RoleRequire(Role.SCHOOL_ADMIN)
    public ModelAndView list(HttpServletRequest request, UserSearchQuery query) {
        User user = RequestUtils.getWebUser(request).getUser();
        query.setSchoolId(user.getSchoolId());
        query = userService.findByQuery(query);

        ModelAndView view = new ModelAndView("modules/user/userList");
        view.addObject("roleList", ROLE_LIST);
        view.addObject("sourceList", UserSource.values());
        view.addObject("query", query);
        return view;
    }

    @RequestMapping(value = "/add", method = RequestMethod.GET)
    @RoleRequire(Role.SCHOOL_ADMIN)
    public ModelAndView addInit(HttpServletRequest request) {
        ModelAndView view = new ModelAndView("modules/user/userEdit");
        User user = new User();
        user.setEnable(true);
        view.addObject("user", user);
        view.addObject("roleList", ROLE_LIST);
        return view;
    }

    @RequestMapping(value = "/edit", method = RequestMethod.GET)
    @RoleRequire(Role.SCHOOL_ADMIN)
    public ModelAndView editInit(HttpServletRequest request, @RequestParam Integer id) {
        User current = RequestUtils.getWebUser(request).getUser();
        User user = userService.findById(id);
        if (user != null && user.getSchoolId().equals(current.getSchoolId())) {
            ModelAndView view = new ModelAndView("modules/user/userEdit");
            view.addObject("user", user);
            if (user.getRole() == Role.SUBJECT_HEADER) {
                view.addObject("subjectCodeString",
                        StringUtils.join(subjectUserService.findSubjectCode(user.getId()), SUBJECT_CODE_SPLIT));
            }
            return view;
        } else {
            return new ModelAndView("redirect:/admin/user/list");
        }
    }

    @Logging(menu = "新增（修改）用户", type = LogType.UPDATE)
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    @RoleRequire(Role.SCHOOL_ADMIN)
    public String save(HttpServletRequest request, Model model, User user,
            @RequestParam(required = false) String subjectCodeString) {
        User current = RequestUtils.getWebUser(request).getUser();
        User previous = null;
        if (user.getId() != null) {
            previous = userService.findById(user.getId());
        }
        if (previous == null) {
            String message = validate(user, subjectCodeString);
            if (message == null) {
                user.setPassword(EncryptUtils.md5(user.getPassword()));
                user.setSchoolId(current.getSchoolId());
                user.setSource(UserSource.INTERNAL);
                user.setCreatedTime(new Date());
                user.setUpdatedTime(new Date());
                userService.save(user);
            } else {
                model.addAttribute("user", user);
                model.addAttribute("roleList", ROLE_LIST);
                model.addAttribute("message", message);
                return "modules/user/userEdit";
            }
        } else if (previous.getSchoolId().equals(current.getSchoolId())) {
            previous.setLoginName(user.getLoginName());
            previous.setName(user.getName());
            if (StringUtils.isNotBlank(user.getPassword())) {
                previous.setPassword(EncryptUtils.md5(user.getPassword()));
            }
            previous.setEnable(user.isEnable());
            String message = validate(previous, subjectCodeString);
            if (message == null) {
                if (previous.getRole() == Role.SUBJECT_HEADER) {
                    subjectUserService.updateByUserId(previous.getId(), getSubjectCodeSet(subjectCodeString));
                }
                previous.setUpdatedTime(new Date());
                userService.save(previous);
            } else {
                model.addAttribute("user", previous);
                model.addAttribute("roleList", ROLE_LIST);
                model.addAttribute("message", message);
                return "modules/user/userEdit";
            }
        }
        return "redirect:/admin/user/list";
    }

    private String validate(User user, String subjectCodeString) {
        String message = null;
        if (StringUtils.isBlank(user.getLoginName())) {
            message = "登录名不能为空";
        } else if (!checkLoginName(user)) {
            message = "登录名不能重复";
        } else if (StringUtils.isBlank(user.getName())) {
            message = "名称不能为空";
        } else if (StringUtils.isBlank(user.getPassword())) {
            message = "密码不能为空";
        } else if (user.getRole() == Role.SUBJECT_HEADER && StringUtils.isBlank(subjectCodeString)) {
            message = "科组长必须绑定科目代码";
        }
        return message;
    }

    private Set<String> getSubjectCodeSet(String subjectCodeString) {
        Set<String> set = new HashSet<>();
        try {
            String[] values = StringUtils.split(subjectCodeString, SUBJECT_CODE_SPLIT);
            for (String value : values) {
                value = StringUtils.trimToNull(value);
                if (value != null) {
                    set.add(value.toUpperCase());
                }
            }
        } catch (Exception e) {

        }
        return set;
    }
}
