package cn.com.qmth.stmms.api.controller;

import cn.com.qmth.stmms.api.exception.ApiException;
import cn.com.qmth.stmms.biz.exam.model.Exam;
import cn.com.qmth.stmms.biz.exam.model.ExamQuestion;
import cn.com.qmth.stmms.biz.exam.model.ExamStudent;
import cn.com.qmth.stmms.biz.exam.query.ExamStudentSearchQuery;
import cn.com.qmth.stmms.biz.exam.service.ExamQuestionService;
import cn.com.qmth.stmms.biz.exam.service.ExamService;
import cn.com.qmth.stmms.biz.exam.service.ExamStudentService;
import cn.com.qmth.stmms.biz.file.service.FileService;
import cn.com.qmth.stmms.biz.utils.ScoreItem;
import cn.com.qmth.stmms.common.annotation.RoleRequire;
import cn.com.qmth.stmms.common.domain.ApiUser;
import cn.com.qmth.stmms.common.enums.*;
import cn.com.qmth.stmms.common.utils.DateUtils;
import cn.com.qmth.stmms.common.utils.RequestUtils;
import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;

import java.text.DecimalFormat;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Controller("coreApiController")
@RequestMapping("/api")
public class CoreController extends BaseApiController {

    private static final int MAX_QUERY_PAGE_SIZE = 100;

    private static final String NULL_PAPER_TYPE_PLACEHOLDER = "#";

    private static final String PAPER_TYPES_REGEX = "[a-zA-Z#]";

    @Autowired
    private ExamService examService;

    @Autowired
    private ExamStudentService studentService;

    @Autowired
    private ExamQuestionService questionService;

    @Autowired
    private FileService fileService;

    @RequestMapping(value = "/exam/save", method = RequestMethod.POST)
    @ResponseBody
    @RoleRequire({ Role.SCHOOL_ADMIN, Role.SCHOOL_DEV })
    public JSONObject examSave(HttpServletRequest request, @RequestParam(required = false) Integer id,
            @RequestParam(required = false) String code, @RequestParam String name,
            @RequestParam(required = false) String examTime) {
        ApiUser user = RequestUtils.getApiUser(request);
        JSONObject result = new JSONObject();
        // 输入字段预处理并初步校验
        code = validate("code", code, false, 32);
        name = validate("name", name, true, 32);
        Date time = DateUtils.parseDate(examTime);
        Exam current = null;
        if (id != null) {
            // 根据id查找考试并校验
            current = examService.findById(id);
            if (current == null) {
                throw ApiException.QUERY_PARAM_ERROR.replaceMessage("id invalid");
            }
        } else if (code != null) {
            // 根据code查找考试并校验
            current = examService.findBySchoolAndCode(user.getSchoolId(), code);
        } else {
            throw ApiException.QUERY_PARAM_ERROR.replaceMessage("id/code both unexists");
        }
        if (current == null) {
            // 新建考试并校验考试时间
            if (time == null) {
                throw ApiException.QUERY_PARAM_ERROR.replaceMessage("examTime invalid");
            }
            current = new Exam();
            current.setCode(code);
            current.setSchoolId(user.getSchoolId());
            current.setType(ExamType.SCAN_IMAGE);
            current.setStatus(ExamStatus.START);
            current.setForbiddenInfo(false);
            current.setObjectiveStatus(ObjectiveStatus.WAITING);
            current.setCreateTime(new Date());
        } else if (!current.getSchoolId().equals(user.getSchoolId()) || current.getStatus() != ExamStatus.START) {
            throw ApiException.EXAM_NOT_ACCESSIBLED;
        }
        current.setName(name);
        if (time != null) {
            current.setExamTime(time);
        }
        current = examService.save(current);
        result.accumulate("id", current.getId());
        result.accumulate("updateTime", DateUtils.formatDateTime(new Date()));
        return result;
    }

    @RequestMapping(value = "/exam/student/save", method = RequestMethod.POST)
    @ResponseBody
    @RoleRequire({ Role.SCHOOL_ADMIN, Role.SCHOOL_DEV })
    public JSONObject examStudentSave(HttpServletRequest request, @RequestParam Integer examId,
            @RequestParam String examNumber, @RequestParam String studentCode, @RequestParam String name,
            @RequestParam String college, @RequestParam String className, @RequestParam String teacher,
            @RequestParam String subjectCode, @RequestParam String subjectName,
            @RequestParam(required = false) String packageCode, @RequestParam(required = false) String paperType,
            @RequestParam(required = false) String examSite, @RequestParam(required = false) String examRoom) {
        ApiUser user = RequestUtils.getApiUser(request);
        Exam exam = examService.findById(examId);
        if (exam == null || !exam.getSchoolId().equals(user.getSchoolId()) || exam.getStatus() != ExamStatus.START) {
            throw ApiException.EXAM_NOT_ACCESSIBLED;
        }
        examNumber = validate("examNumber", examNumber, true, 64);
        ExamStudent student = studentService.findByExamIdAndExamNumber(examId, examNumber);
        if (student == null) {
            // 新建考生
            student = new ExamStudent();
            student.setExamId(examId);
            student.setSchoolId(exam.getSchoolId());
            student.setExamNumber(examNumber);
            student.setPackageCode(validate("packageCode", packageCode, false, 64));
            student.setStudentCode(validate("studentCode", studentCode, true, 64));
            student.setName(validate("name", name, true, 32));
            student.setSubjectCode(validate("subjectCode", subjectCode, true, 32));
            student.setSubjectName(validate("subjectName", subjectName, true, 32));
            student.setCollege(validate("college", college, true, 32));
            student.setClassName(validate("className", className, true, 32));
            student.setTeacher(validate("teacher", teacher, true, 32));
            validate("paperType", paperType, false, 16);
            if (paperType != null && paperType.matches(PAPER_TYPES_REGEX)) {
                student.setPaperType(paperType);
            } else {
                student.setPaperType(NULL_PAPER_TYPE_PLACEHOLDER);
            }
            student.setExamSite(validate("examSite", examSite, false, 32));
            student.setExamRoom(validate("examRoom", examRoom, false, 32));
            student.setAbsent(false);
            student.setManualAbsent(false);
            student.setUpload(false);
            student.setException(false);
            student.setBreach(false);
            student.setSliceCount(0);
            student.setSheetCount(0);
            student.setObjectiveScore(0d);
            student.setSubjectiveScore(0d);
            student.setSubjectiveStatus(SubjectiveStatus.UNMARK);
        } else {
            // 更新现有考生
            name = validate(name, true, 32);
            if (name != null) {
                student.setName(name);
            }
            studentCode = validate(studentCode, true, 64);
            if (studentCode != null) {
                student.setStudentCode(studentCode);
            }
            subjectCode = validate(subjectCode, true, 32);
            subjectName = validate(subjectName, true, 32);
            if (subjectCode != null && subjectName != null) {
                student.setSubjectCode(subjectCode);
                student.setSubjectName(subjectName);
            }
            paperType = validate(paperType, true, 16);
            if (paperType != null && paperType.matches(PAPER_TYPES_REGEX)) {
                student.setPaperType(paperType);
            }
            college = validate(college, true, 32);
            if (college != null) {
                student.setCollege(college);
            }
            className = validate(className, true, 32);
            if (className != null) {
                student.setClassName(className);
            }
            teacher = validate(teacher, true, 32);
            if (teacher != null) {
                student.setTeacher(teacher);
            }
            examSite = validate(examSite, false, 32);
            if (examSite != null) {
                student.setExamSite(examSite);
            }
            examRoom = validate(examRoom, false, 32);
            if (examRoom != null) {
                student.setExamRoom(examRoom);
            }
        }
        studentService.save(student);
        JSONObject result = new JSONObject();
        result.accumulate("updateTime", DateUtils.formatDateTime(new Date()));
        return result;
    }

    @RequestMapping(value = "/student/query", method = RequestMethod.POST)
    @ResponseBody
    @RoleRequire({ Role.SCHOOL_ADMIN, Role.SCHOOL_DEV })
    public JSONArray studentQuery(HttpServletRequest request, @RequestParam String studentCode,
            @RequestParam(required = false) String subjectCode, @RequestParam(required = false) Date minExamTime) {
        ApiUser user = RequestUtils.getApiUser(request);
        JSONArray array = new JSONArray();
        studentCode = StringUtils.trimToNull(studentCode);
        subjectCode = StringUtils.trimToNull(subjectCode);
        if (studentCode == null) {
            throw ApiException.QUERY_PARAM_ERROR.replaceMessage("studentCode invalid");
        }
        ExamStudentSearchQuery query = new ExamStudentSearchQuery();
        query.setSchoolId(user.getSchoolId());
        query.setStudentCode(studentCode);
        query.setSubjectCode(subjectCode);
        query.orderByExamIdDesc();
        query = studentService.findByQuery(query);
        for (ExamStudent student : query.getResult()) {
            Exam exam = examService.findById(student.getExamId());
            if (minExamTime != null && minExamTime.before(exam.getExamTime())) {
                JSONObject obj = new JSONObject();
                obj.accumulate("examId", exam.getId());
                obj.accumulate("examCode", exam.getCode());
                obj.accumulate("examTime", DateUtils.formatDateTime(exam.getExamTime()));
                obj.accumulate("examNumber", student.getExamNumber());
                obj.accumulate("name", student.getName());
                obj.accumulate("subjectCode", student.getSubjectCode());
                obj.accumulate("subjectName", student.getSubjectName());
                obj.accumulate("examSite", StringUtils.trimToEmpty(student.getExamSite()));
                obj.accumulate("examRoom", StringUtils.trimToEmpty(student.getExamRoom()));
                array.add(obj);
            }
        }
        return array;
    }

    @RequestMapping(value = "/exam/student/score", method = RequestMethod.POST)
    @ResponseBody
    @RoleRequire({ Role.SCHOOL_ADMIN, Role.SCHOOL_DEV })
    public JSONArray examStudentScore(HttpServletRequest request, @RequestParam(required = false) Integer examId,
            @RequestParam(required = false) String examCode, @RequestParam(required = false) String examNumber,
            @RequestParam(required = false) String studentCode, @RequestParam(required = false) String subjectCode,
            @RequestParam(required = false) String college, @RequestParam(required = false) String className,
            @RequestParam(required = false) String teacher,
            @RequestParam(required = false, defaultValue = "1") Integer pageNumber,
            @RequestParam(required = false, defaultValue = "20") Integer pageSize) {
        ApiUser user = RequestUtils.getApiUser(request);
        if (examId == null && examCode == null) {
            throw ApiException.QUERY_PARAM_ERROR.replaceMessage("examId or examCode is required");
        }
        Exam exam = examService.findById(examId);
        if (exam == null) {
            exam = examService.findBySchoolAndCode(user.getSchoolId(), examCode);
        }
        if (exam == null) {
            throw ApiException.QUERY_PARAM_ERROR.replaceMessage("exam unexists");
        }
        if (!exam.getSchoolId().equals(user.getSchoolId())) {
            throw ApiException.EXAM_NOT_ACCESSIBLED;
        }
        JSONArray array = new JSONArray();
        ExamStudentSearchQuery query = new ExamStudentSearchQuery();
        query.setExamId(exam.getId());
        query.setExamNumber(StringUtils.trimToNull(examNumber));
        query.setStudentCode(StringUtils.trimToNull(studentCode));
        query.setSubjectCode(StringUtils.trimToNull(subjectCode));
        query.setCollege(StringUtils.trimToNull(college));
        query.setClassName(StringUtils.trimToNull(className));
        query.setTeacher(StringUtils.trimToNull(teacher));
        query.setPageNumber(Math.max(1, pageNumber));
        query.setPageSize(Math.min(MAX_QUERY_PAGE_SIZE, pageSize));
        query = studentService.findByQuery(query);
        DecimalFormat df = new DecimalFormat("####.###");
        for (ExamStudent student : query.getResult()) {
            JSONObject obj = new JSONObject();
            obj.accumulate("examId", exam.getId());
            obj.accumulate("examCode", StringUtils.trimToEmpty(exam.getCode()));
            obj.accumulate("examNumber", student.getExamNumber());
            obj.accumulate("studentCode", student.getStudentCode());
            obj.accumulate("name", student.getName());
            obj.accumulate("subjectCode", student.getSubjectCode());
            obj.accumulate("subjectName", student.getSubjectName());
            obj.accumulate("paperType", StringUtils.trimToEmpty(student.getPaperType()));
            obj.accumulate("status", getStatus(student));
            obj.accumulate("totalScore", df.format(student.getTotalScore()));
            obj.accumulate("objectiveScore", df.format(student.getObjectiveScore()));
            obj.accumulate("subjectiveScore", df.format(student.getSubjectiveScore()));
            // obj.accumulate("objectiveScoreDetail",
            // StringUtils.trimToEmpty(student.getObjectiveScoreList()));
            // obj.accumulate("subjectiveScoreDetail",
            // StringUtils.trimToEmpty(student.getSubjectiveScoreList()));
            // 构造客观题得分明细
            JSONArray objective = new JSONArray();
            List<ScoreItem> scoreList = student.getScoreList(true);
            List<ExamQuestion> questionList = questionService.findByExamAndSubjectAndObjectiveAndPaperType(
                    student.getExamId(), student.getSubjectCode(), true, student.getPaperType());
            int i = 0;
            for (ScoreItem item : scoreList) {
                i++;
                if (questionList.size() < i) {
                    break;
                }
                ExamQuestion question = questionList.get(i - 1);
                if (question.getTotalScore() == null || question.getTotalScore() == 0) {
                    continue;
                }
                JSONObject detail = new JSONObject();
                detail.accumulate("mainNumber", question.getMainNumber());
                detail.accumulate("subNumber", question.getSubNumber());
                detail.accumulate("score", df.format(item.getScore()));
                detail.accumulate("answer", StringUtils.trimToEmpty(item.getAnswer()));
                objective.add(detail);
            }
            obj.accumulate("objectiveScoreDetail", objective);
            // 构造主观题得分明细
            JSONArray subjective = new JSONArray();
            scoreList = student.getScoreList(false);
            questionList = questionService.findByExamAndSubjectAndObjective(student.getExamId(),
                    student.getSubjectCode(), false);
            i = 0;
            for (ScoreItem item : scoreList) {
                i++;
                if (questionList.size() < i) {
                    break;
                }
                ExamQuestion question = questionList.get(i - 1);
                if (question.getTotalScore() == null || question.getTotalScore() == 0) {
                    continue;
                }
                JSONObject detail = new JSONObject();
                detail.accumulate("mainNumber", question.getMainNumber());
                detail.accumulate("subNumber", question.getSubNumber());
                detail.accumulate("score", df.format(item.getScore()));
                subjective.add(detail);
            }
            obj.accumulate("subjectiveScoreDetail", subjective);
            // 构造原图下载地址
            if (student.isUpload()) {
                obj.accumulate(
                        "sheetUrls",
                        fileService
                                .getSheetUris(student.getExamId(), student.getExamNumber(), 1, student.getSheetCount())
                                .stream().map(uri -> fileService.getFileServer().concat(uri))
                                .collect(Collectors.toList()));
            } else {
                obj.accumulate("sheetUrls", new JSONArray());
            }
            array.add(obj);
        }
        return array;
    }

    private int getStatus(ExamStudent student) {
        if (student.isBreach()) {
            return 3;
        }
        if (!student.isUpload() || student.isAbsent() || student.isManualAbsent()) {
            return 2;
        }
        return 1;
    }

    @RequestMapping(value = "/exam/student/count", method = RequestMethod.POST)
    @ResponseBody
    @RoleRequire({ Role.SCHOOL_ADMIN, Role.SCHOOL_DEV })
    public JSONObject examStudentCount(HttpServletRequest request, @RequestParam(required = false) Integer examId,
            @RequestParam(required = false) String examCode, @RequestParam(required = false) String examNumber,
            @RequestParam(required = false) String studentCode, @RequestParam(required = false) String subjectCode,
            @RequestParam(required = false) String college, @RequestParam(required = false) String className,
            @RequestParam(required = false) String teacher) {
        ApiUser user = RequestUtils.getApiUser(request);
        if (examId == null && examCode == null) {
            throw ApiException.QUERY_PARAM_ERROR.replaceMessage("examId or examCode is required");
        }
        Exam exam = examService.findById(examId);
        if (exam == null) {
            exam = examService.findBySchoolAndCode(user.getSchoolId(), examCode);
        }
        if (exam == null) {
            throw ApiException.QUERY_PARAM_ERROR.replaceMessage("exam unexists");
        }
        if (!exam.getSchoolId().equals(user.getSchoolId())) {
            throw ApiException.EXAM_NOT_ACCESSIBLED;
        }
        ExamStudentSearchQuery query = new ExamStudentSearchQuery();
        query.setExamId(exam.getId());
        query.setExamNumber(StringUtils.trimToNull(examNumber));
        query.setStudentCode(StringUtils.trimToNull(studentCode));
        query.setSubjectCode(StringUtils.trimToNull(subjectCode));
        query.setCollege(StringUtils.trimToNull(college));
        query.setClassName(StringUtils.trimToNull(className));
        query.setTeacher(StringUtils.trimToNull(teacher));
        long count = studentService.countByQuery(query);
        JSONObject result = new JSONObject();
        result.accumulate("totalCount", count);
        return result;
    }
}
